diff -u -r asterisk-1.2.11.dist/channels/chan_iax2.c asterisk-1.2.11/channels/chan_iax2.c
--- asterisk-1.2.11.dist/channels/chan_iax2.c	2006-08-05 07:08:50.000000000 +0200
+++ asterisk-1.2.11/channels/chan_iax2.c	2006-09-03 15:47:12.217469680 +0200
@@ -8194,16 +8194,36 @@
 			if (sock) {
 				sockfd = ast_netsock_sockfd(sock);
 				nonlocal = 0;
+			} else {
+				/* Stevenson! <stevens@stevens.it> - 20060903 */
+				unsigned int orig_saddr = sin.sin_addr.s_addr;
+				/* INADDR_ANY matches anyway! */
+				sin.sin_addr.s_addr = INADDR_ANY;
+				if (ast_netsock_find(netsock, &sin)) {
+					sin.sin_addr.s_addr = orig_saddr;
+					sock = ast_netsock_bind(netsock, io, srcaddr, port, tos, socket_read, NULL);
+					if (sock) {
+						sockfd = ast_netsock_sockfd(sock);
+						ast_netsock_unref(sock);
+						nonlocal = 0;
+					} else {
+						nonlocal = 2;
+					}
+				}
 			}
 		}
 	}
 		
 	peer->sockfd = sockfd;
 
-	if (nonlocal) {
+	if (nonlocal == 1) {
 		ast_log(LOG_WARNING, "Non-local or unbound address specified (%s) in sourceaddress for '%s', reverting to default\n",
 			srcaddr, peer->name);
 		return -1;
+        } else if (nonlocal == 2) {
+		ast_log(LOG_WARNING, "Unable to bind to sourceaddress '%s' for '%s', reverting to default\n",
+			srcaddr, peer->name);
+		return -1;
 	} else {
 		ast_log(LOG_DEBUG, "Using sourceaddress %s for '%s'\n", srcaddr, peer->name);
 		return 0;
diff -u -r asterisk-1.2.11.dist/netsock.c asterisk-1.2.11/netsock.c
--- asterisk-1.2.11.dist/netsock.c	2005-11-29 19:24:39.000000000 +0100
+++ asterisk-1.2.11/netsock.c	2006-09-03 15:44:12.937724344 +0200
@@ -128,7 +128,8 @@
 	char iabuf[INET_ADDRSTRLEN];
 	
 	struct ast_netsock *ns;
-	
+	const int reuseFlag = 1;
+
 	/* Make a UDP socket */
 	netsocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
 	
@@ -136,6 +137,9 @@
 		ast_log(LOG_ERROR, "Unable to create network socket: %s\n", strerror(errno));
 		return NULL;
 	}
+	if (setsockopt(netsocket, SOL_SOCKET, SO_REUSEADDR, (char *)&reuseFlag, sizeof reuseFlag) < 0) {
+		ast_log(LOG_WARNING, "Error setting SO_REUSEADDR on sockfd '%d'\n", netsocket);
+	}
 	if (bind(netsocket,(struct sockaddr *)bindaddr, sizeof(struct sockaddr_in))) {
 		ast_log(LOG_ERROR, "Unable to bind to %s port %d: %s\n", ast_inet_ntoa(iabuf, sizeof(iabuf), bindaddr->sin_addr), ntohs(bindaddr->sin_port), strerror(errno));
 		close(netsocket);
